<?php

if (!defined('ABSPATH')) {
    exit;
}

class WC_BSPB extends WC_Payment_Gateway
{
    public function __construct()
    {
        $this->id = 'bspb';

        $this->init_form_fields();
        $this->init_settings();

        $this->enabled = $this->get_option('enabled');
        $this->test = ($this->get_option('test') == 'yes') ? true : false;
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description_site');
        $this->merchant = $this->get_option('merchant');
        $this->order_type = $this->get_option('order_type');
        $this->currency = $this->get_option('currency');
        $this->description_ie = $this->get_option('description_ie');
        $this->language = $this->get_option('language');

        $this->certName1 = __DIR__ . '/' . $this->get_option('cert1_name');
        $this->certName2 = __DIR__ . '/' . $this->get_option('cert2_name');

        if (strlen($this->get_option('cert1')) > 0) 
            $this->form_fields['cert1']['description'] = 'Файл ' . $this->get_option('cert1') . ' успешно загружен!';

        if (strlen($this->get_option('cert2')) > 0) 
            $this->form_fields['cert2']['description'] = 'Файл ' . $this->get_option('cert2') . ' успешно загружен!';

        add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
        add_action('woocommerce_api_wc_' . $this->id, array($this, 'result_pay'));
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
    }

    public function process_admin_options() {
        if (array_key_exists('woocommerce_bspb_cert1', $_FILES)
                && array_key_exists('tmp_name', $_FILES['woocommerce_bspb_cert1'])
                && array_key_exists('size', $_FILES['woocommerce_bspb_cert1'])
                && $_FILES['woocommerce_bspb_cert1']['size']
            ) {
            $_POST['woocommerce_bspb_cert1'] = $_FILES['woocommerce_bspb_cert1']['name'];
            $fileNameCert1 = substr(md5(time()), 0, 16) . '.pem';
            $_POST['woocommerce_bspb_cert1_name'] = $fileNameCert1;
            move_uploaded_file($_FILES['woocommerce_bspb_cert1']['tmp_name'], __DIR__ . '/' . $fileNameCert1);
            unlink($_FILES['woocommerce_bspb_cert1']['tmp_name']);
            unset($_FILES['woocommerce_bspb_cert1']);
        } else {
            $_POST['woocommerce_bspb_cert1'] = $this->get_option('cert1');
            $_POST['woocommerce_bspb_cert1_name'] = $this->get_option('cert1_name');
        }
        
        if (array_key_exists('woocommerce_bspb_cert2', $_FILES)
                && array_key_exists('tmp_name', $_FILES['woocommerce_bspb_cert2'])
                && array_key_exists('size', $_FILES['woocommerce_bspb_cert2'])
                && $_FILES['woocommerce_bspb_cert2']['size']
            ) {
            $_POST['woocommerce_bspb_cert2'] = $_FILES['woocommerce_bspb_cert2']['name'];
            $fileNameCert2 = substr(md5(time()), 0, 16) . '.key';
            $_POST['woocommerce_bspb_cert2_name'] = $fileNameCert2;
            move_uploaded_file($_FILES['woocommerce_bspb_cert2']['tmp_name'], __DIR__ . '/' . $fileNameCert2);
            unlink($_FILES['woocommerce_bspb_cert2']['tmp_name']);
            unset($_FILES['woocommerce_bspb_cert2']);
        } else {
            $_POST['woocommerce_bspb_cert2'] = $this->get_option('cert2');
            $_POST['woocommerce_bspb_cert2_name'] = $this->get_option('cert2_name');
        }
        parent::process_admin_options();
    }

    public function process_payment($order_id)
    {
        $order = new WC_Order($order_id);
        return array(
            'result' => 'success',
            'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(wc_get_page_id('pay')))),
        );
    }

    public function receipt_page($order_id)
    {
        $order = new WC_Order($order_id);

        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'CreateOrder');
        $xmlRequest->addChild('Language', $this->language);
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('OrderType', $this->order_type);
        $xmlOrder->addChild('Merchant', $this->merchant);
        $xmlOrder->addChild('Amount', ((float)$order->order_total * 100));
        $xmlOrder->addChild('Currency', $this->currency);
        $xmlOrder->addChild('Description', $this->description_ie);
        
        $resultUrl = $this->getResultUrl($infoPay);
        $xmlOrder->addChild('ApproveURL', $resultUrl['ApproveURL']);
        $xmlOrder->addChild('CancelURL', $resultUrl['CancelURL']);
        $xmlOrder->addChild('DeclineURL', $resultUrl['DeclineURL']);
        
        $xmlAddParams = $xmlOrder->addChild('AddParams');
        $xmlAddParams->addChild('ShopOrderNum', $order_id);

        $strResponse = $this->sendPostXml($this->getPayUrl(), $xml->asXML(), $this->certName1, $this->certName2);
        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            $response = $this->XmlToArray($strResponse['data']);
            if ($response['Response']['Status'] == '00') {
                $urlToBank = $response['Response']['Order']['URL'] . "?ORDERID=" . $response['Response']['Order']['OrderID']."&SESSIONID=" . $response['Response']['Order']['SessionID'];
                wp_redirect($urlToBank);
            } else {
                $errors[] = 'Error: status ' . $response['Response']['Status'];
            }
        } else {
            $errors[] = 'Error: ' . $strResponse['error'];
        }

        foreach($errors as $key => $err) {
            echo '<p>' . __('Ошибка #' . key . ': ' . $err, 'woocommerce') . '</p>';
        }
    }

    public function result_pay() 
    {
        if (strlen($_REQUEST['xmlmsg']) > 0) {
            $data = $this->XmlToArray($_REQUEST['xmlmsg']);
            if (array_key_exists('Message', $data)) {
                $orderId = $data['Message']['OrderID'];
                $sessionId = $data['Message']['SessionID'];
            } else {
                $orderId = $data['OrderID'];
                $sessionId = $data['SessionId'];
            }

            $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
            $xmlRequest = $xml->addChild('Request');
            $xmlRequest->addChild('Operation', 'GetOrderInformation');
            $xmlRequest->addChild('Language', $this->language);
            $xmlOrder = $xmlRequest->addChild('Order');
            $xmlOrder->addChild('Merchant', $this->merchant);
            $xmlOrder->addChild('OrderID', $orderId);
            $xmlRequest->addChild('SessionID', $sessionId);
            $xmlRequest->addChild('ShowParams', 'true');
            $xmlRequest->addChild('ShowOperations', 'true');
            $xmlRequest->addChild('ClassicView', 'true');

            $strResponse = $this->sendPostXml($this->getPayUrl(), $xml->asXML(), $this->certName1, $this->certName2);

            if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
                $dataInit = $this->XmlToArray($strResponse['data']);
                if ($dataInit && $dataInit['Response']['Status'] == '00') {
                    $order = new \WC_Order($dataInit['Response']['Order']['row']['OrderParams']['row']['0']['VAL']);
                    if ($dataInit['Response']['Order']['row']['Orderstatus'] == 'APPROVED') {
                        if ((float)$order->order_total * 100 == (float)$dataInit['Response']['Order']['row']['Amount']) {
                            $order->update_status('processing', __('Платеж успешно оплачен', 'woocommerce'));
                            global $woocommerce;
                            $woocommerce->cart->empty_cart();
                            wp_redirect($this->get_return_url($order));
                            exit;
                        } else {
                            $order->update_status('failed', __('Платеж не оплачен', 'woocommerce'));
                            wp_redirect($order->get_cancel_order_url());
                            exit;
                        }
                    } else {
                        $order->update_status('failed', __('Платеж не оплачен', 'woocommerce'));
                        wp_redirect($order->get_cancel_order_url());
                        exit;
                    }
                }
            }
        }
        exit;
    }

    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Активность', 'woocommerce'),
                'type' => 'checkbox',
                'label' => __('Включен', 'woocommerce'),
                'default' => 'yes',
            ),
            'test' => array(
                'title' => __('Тест режим', 'woocommerce'),
                'type' => 'checkbox',
                'label' => __('Включен', 'woocommerce'),
                'description' => __('В этом режиме плата за товар не снимается.', 'woocommerce'),
                'default' => 'yes',
            ),

            'title' => array(
                'title' => __('Название', 'woocommerce'),
                'type' => 'text',
                'description' => __('Название платежной системы на странице оплаты', 'woocommerce'),
                'default' => __('Оплата картой', 'woocommerce'),
            ),
            'description_site' => array(
                'title' => __('Описание заказа для сайта', 'woocommerce'),
                'type' => 'textarea',
                'description' => __('Описание платежной системы на странице оплаты', 'woocommerce'),
                'default' => 'Вы можете оплатить свой заказ',
            ),

            'merchant' => array(
                'title' => __('Merchant', 'woocommerce'),
                'type' => 'text',
                'description' => __('Идентификатор магазина, который выдается банком', 'woocommerce'),
                'default' => '',
            ),
            'cert1' => array(
                'title' => __('Файл *.rem', 'woocommerce'),
                'type' => 'file',
                'default' => '',
            ),
            'cert2' => array(
                'title' => __('Файл *.key', 'woocommerce'),
                'type' => 'file',
                'default' => '',
            ),
            'order_type' => array(
                'title' => __('Тип транзакции', 'woocommerce'),
                'type' => 'select',
                'default' => '0',
                'options' => array(
                    'Purchase' => 'Purchase',
                    'Payments' => 'Payments',
                ),
            ),
            'currency' => array(
                'title' => __('Валюта платежа', 'woocommerce'),
                'type' => 'select',
                'default' => '643',
                'options' => array(
                    '643' => __('рубли', 'woocommerce'),
                ),
            ),
            'description_ie' => array(
                'title' => __('Описание заказа для банка', 'woocommerce'),
                'type' => 'text',
                'description' => __('Обязательный параметр', 'woocommerce'),
                'default' => 'Покупка в интернет-магазине',
            ),
            'language' => array(
                'title' => __('Язык интерфейса', 'woocommerce'),
                'type' => 'select',
                'default' => 'RU',
                'options' => array(
                    'RU' => __('русский', 'woocommerce'),
                    'EN' => __('английский', 'woocommerce'),
                ),
            ),

            'cert1_name' => array(
                'type' => 'hidden',
            ),
            'cert2_name' => array(
                'type' => 'hidden',
            ),
        );
    }
    
    private function getPayUrl()
    {
        if ($this->test) {
            return 'https://mpitest.bspb.ru:5443/Exec';
        } else {
            return 'https://mpi.bspb.ru:5443/Exec';
        }
    }

    private function sendPostXml($url, $xml, $pathCertPem, $pathCertKey)
    {
        $error = '';
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RESOLVE, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSLCERT, $pathCertPem);
        curl_setopt($ch, CURLOPT_SSLKEY, $pathCertKey);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        if (!$answer = curl_exec($ch)) {
            $error = 'Error with connection: ' . curl_errno($ch);
        }
        curl_close($ch);

        return array(
            'data' => $answer,
            'error' => $error
        );
    }

    private function XmlToArray($strXml)
    {
        $strXml = str_replace("\\\"", "'", $strXml);
        $json = json_encode(simplexml_load_string($strXml));
        return json_decode($json, true);
    }

    private function getResultUrl($infoPay)
    {
        $url = get_site_url() . '/wc-api/WC_BSPB/';

        return array(
            'ApproveURL' => $url,
            'CancelURL' => $url,
            'DeclineURL' => $url,
        );
    }
}
